<?php

namespace SM\Service;

use SM\Factory\SmPropostasImpostosFactory;
use Teknisa\Libs\Util\Environment;
use SM\Service\FindService;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class SmPropostasImpostosService
{
    protected $connection;
    protected $impostoFactory;
    protected $environment;

    protected $findService;

    protected $newCode;

    public function __construct(SmPropostasImpostosFactory $impostoFactory, Environment $environment, FindService $findService, NewCode $newCode, EntityManager $entityManager)
    {
        $this->impostoFactory = $impostoFactory;
        $this->environment = $environment;
        $this->findService = $findService;
        $this->newCode = $newCode;
        $this->connection = $entityManager->getConnection();
    }

    public function getImpostoproposta()
    {
        return $this->impostoFactory->getImpostoproposta();
    }

    public function getTabComposicoesProposta($params)
    {
        $nrtipoorcamento = $params['NRTIPOORCAMENTO'] ?? null;
        $nrcomposicao = $params['NRCOMPOSICAO'] ?? null;
    
        return $this->impostoFactory->getTabComposicoesProposta($params, $nrtipoorcamento, $nrcomposicao);
    }
    
    public function getComposicoesProposta($params)
    {
        return $this->impostoFactory->getComposicoesProposta($params);
    }

    public function getVariacaoTipoMedida($parametro)
    {
        return $this->impostoFactory->getVariacaoTipoMedida($parametro);
    }

    public function saveTabImposto($params)
    {
        $this->connection->beginTransaction();
    
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            
            foreach ($params["DRG"] as $row) {
                if ($row["NRIMPOSTOSERV"] != 0) {
                    $parametros = array(
                        "CDIMPOSTO" => $row["CDIMPOSTO"],
                        "VRPRECIMPOSTO" => $row["VRPRECIMPOSTO"],
                        "NRCOMPSERVICO" => isset($row["NRCOMPSERVICO"]),
                        "NRORG" => $nrorg,
                        "NRORGULTATU" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => "S",
                        "NRIMPOSTOSERV" => $row["NRIMPOSTOSERV"],
                    );
                    
                    $this->impostoFactory->updateTabImposto($parametros);
                } else {
                    $NRIMPOSTOSERV = $this->newCode->getCodeByTableName("FCT_RELACIMPOSTOSERV", $nrorg, 1, 10, true);
                    
                    $parametros = array(
                    "NRIMPOSTOSERV" =>  $NRIMPOSTOSERV,
                    "CDIMPOSTO" =>     $row["CDIMPOSTO"],
                    "VRPRECIMPOSTO" => $row["VRPRECIMPOSTO"],
                    "NRCOMPSERVICO" => isset($row["NRCOMPSERVICO"]),
                    "NRORG" => $nrorg,
                    "IDATIVO" => "S",
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador
                    );
                    
                    $this->impostoFactory->saveTabImposto($parametros);
                }
            }
            
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }    

    public function saveComposicaoServicoTab($params)
    {
        $this->connection->beginTransaction();
    
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
    
            foreach ($params["DRG"] as $row) {
                $existe = $this->impostoFactory->verificaSeExiste($row);
                if (empty($existe)) {  
                    $NRIMPOSTOSERV = $this->newCode->getCodeByTableName("FCT_RELACIMPOSTOSERV", $nrorg, 1, 10, true);
                    
                    $parametros = [
                        "NRIMPOSTOSERV" => $NRIMPOSTOSERV,
                        "CDIMPOSTO" => $row["CDIMPOSTO"],
                        "NRCOMPSERVICO" => $row["NRCOMPSERVICO"],
                        "NRORG" => $nrorg,
                        "IDATIVO" => "S",
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador
                    ];
                
                    $this->impostoFactory->saveTabComposicao($parametros);
                } else {  
                    $parametros = [
                        "CDIMPOSTO" => $row["CDIMPOSTO"],
                        "NRCOMPSERVICO" => $row["NRCOMPSERVICO"],
                        "NRORG" => $nrorg,
                        "NRORGULTATU" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => "S",
                        "NRIMPOSTOSERV" => $row["NRIMPOSTOSERV"],
                    ];
                    $this->impostoFactory->updateTabComposicao($parametros);                
                }
            }
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }    
    
    public function deleteImposto($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->impostoFactory->deleteImposto($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteImpostoComposicaoServ($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->impostoFactory->deleteImpostoComposicaoServ($retorno);

            $this->connection->commit();
            return 'ok';
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

}