import {
    Http, IEventParam, Metadata,
} from '@zeedhi/core';
import {
    DialogService, LoadingService, Modal, Select, Tabs, TextInput,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class SmPropostasDespesasController {
    public tekgridVisivel: boolean = true;

    public currentRow: any = {};

    public viewTab: boolean = false;

    public tituloHeaderFrameOperadoresEquipes: String = '';

    private ModalFiltro!: Modal;

    get textFrame() {
        return this.currentRow.DSDESPESA;
    }

    get formVisivel() {
        return !this.tekgridVisivel;
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ CDDESPESA: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`DSDESPESA_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public addNewRowComposi({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ CDRELACSERVICODESP: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NRCOMPSERVICO_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public addNewRowFormPreco({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ CDRELACFORMDESP: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NRFORMPRECOSERVICO_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    private clickTimeout: any = null;

    private clicks: number = 0;

    public rowClick({ row, component }: any) {
        this.clicks += 1;
    
        if (this.clicks === 1) {
            this.clickTimeout = setTimeout(() => {
                if (!component.editing) {
                    this.currentRow = row;
                    this.tekgridVisivel = false;
                    const tabs = Metadata.getInstance<Tabs>('tabsInformation');
                    if (tabs.activeTab !== 0) {
                        tabs.activeTab = 0;
                    }
                    setTimeout(() => {
                        this.setFilterComposi();
                        this.setFilterFormpreco();
                    }, 100);
                }
    
                this.clicks = 0;
            }, 300);
        } else if (this.clicks === 2) {
            clearTimeout(this.clickTimeout);
            component.editing(row);
            this.clicks = 0;
        }
    }

    public setFilterFormpreco() {
        const gridComposicao = Metadata.getInstance<TekGrid>('tab-formPreco');
        gridComposicao.datasource.setFilter({
            CDDESPESA: this.currentRow.CDDESPESA,
        });
    }

    public setFilterComposi() {
        const gridComposicao = Metadata.getInstance<TekGrid>('tab-composi');
        gridComposicao.datasource.setFilter({
            CDDESPESA: this.currentRow.CDDESPESA,
        });
    }

    public abrirFrameAdd() {
        this.tekgridVisivel = false;
        const tabs = Metadata.getInstance<Tabs>('tabsInformation');
        tabs.activeTab = 0;
        Metadata.getInstance<Select>('NRCOMPSERVICO').value = null;
    }

    public verificaTab() {
        const tabs = Metadata.getInstance<Tabs>('tabsInformation');
        const tabAtiva = tabs.activeTab;

        if (tabs.tabs[tabAtiva].name === 'tab_variacao') {
            this.carregarDadosAbaVariacao();
        }
    }

    public voltarTekGrid() {
        this.tekgridVisivel = true;
    }

    public carregarDadosAbaVariacao() {
        const cdtpmedidaserv = this.currentRow.NRIMPOSTOSERV;
        try {
            const grid = Metadata.getInstance<TekGrid>('tabcomposi');
            grid.datasource.setFilter({
                CDIMPOSTO: cdtpmedidaserv,
            });
        } catch (e) {
            // console.log(e);
        }
    }

    public saveDespesas({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const sendObject = {
                DRG: component.getEditedRows(),
            };
            Http.post('/saveDespesas', sendObject).then(() => {
                component.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
                LoadingService.hide();
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    /* eslint-disable */
    public saveComposiServ({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const editedRows = component.getEditedRows().map(row => ({
                ...row, CDDESPESA: this.currentRow.CDDESPESA,
            }));
            const sendObject = {
                DRG: editedRows
            };
            Http.post('/saveComposicaoServicosDespesas', sendObject,)
                .then(() => {
                    component.datasource.get();
                    Utilities.showSuccessMessage();
                })
                .catch((error: any) => {
                    Utilities.showErrorMessage(error);
                })
                .finally(() => {
                    component.cancelEditedRows();
                    LoadingService.hide();
                });
        }
    }

    public saveFormPreco({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const editedRows = component.getEditedRows().map(row => ({
                ...row, CDDESPESA: this.currentRow.CDDESPESA,
            }));
            const sendObject = {
                DRG: editedRows
            };
            Http.post('/saveRelacFormDesp', sendObject)
                .then(() => {
                    component.datasource.get();
                    Utilities.showSuccessMessage();
                })
                .catch((error: any) => {
                    Utilities.showErrorMessage(error);
                })
                .finally(() => {
                    component.cancelEditedRows();
                    LoadingService.hide();
                });
        }
    }
    /* eslint-enable */

    public showDialogWithButtonsGrid({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'SmPropostasDespesasController.deleteInvest');
    }

    public deleteInvest() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-investimento');
        LoadingService.show();
        Http.post('/deleteDespesas', {
            NRS: grid.selectedRows.map((element: any) => element.CDDESPESA),
        }).then(() => {
            grid.selectedRows = [];
            grid.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public showDialogBeforeDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'SmPropostasDespesasController.deleteComposi');
    }

    public deleteComposi() {
        const grid = Metadata.getInstance<TekGrid>('tab-composi');
        LoadingService.show();
        Http.post('/deleteRelacCompDespesas', {
            NRS: grid.selectedRows.map((element: any) => element.CDRELACSERVICODESP),
        }).then(() => {
            grid.selectedRows = [];
            grid.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public showDialogFormBeforeDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'SmPropostasDespesasController.deleteFormacaoPreco');
    }

    public deleteFormacaoPreco() {
        const grid = Metadata.getInstance<TekGrid>('tab-formPreco');
        LoadingService.show();
        Http.post('/deleteFormacaoPreco', {
            NRS: grid.selectedRows.map((element: any) => element.CDRELACFORMDESP),
        }).then(() => {
            grid.selectedRows = [];
            grid.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
